import logging

from tempfile import NamedTemporaryFile
from subprocess import run, CalledProcessError, Popen, PIPE

logger = logging.getLogger(__name__)


def send(ssh_host: str,
         target_dir: str,
         temp_archive: NamedTemporaryFile):
    """Sends a temporary archive to a server, then runs deployment there.
    """
    logger.info("Send started:")
    transfer_file(ssh_host, "temp_pictures", temp_archive)
    temp_archive.close()
    ssh_process = spawn_ssh_process(ssh_host)
    unpack(ssh_process, "temp_pictures")
    move_to_final(ssh_process, target_dir, "temp_pictures")
    logger.info("Send complete.\n")


def transfer_file(ssh_host: str,
                  target_temp_dir: str,
                  temp_archive: NamedTemporaryFile):
    source = temp_archive.name
    dest = f"{ssh_host}:{target_temp_dir}"
    print(dest)

    logger.debug(f"\t\t Running: rsync ... {source} {dest}")

    logger.info("\t- Initiating file transfer")
    try:
        run(["/usr/bin/rsync", "-rt", "-e", "ssh", source, dest],
            capture_output=True,
            check=True)
    except CalledProcessError:
        transfer_recovery(target_temp_dir, temp_archive)
        return
    logger.info("\t\t File transfer complete")


def transfer_recovery(target_temp_dir: str,
                      temp_archive: NamedTemporaryFile):
    logger.error("\tTransfer failed. Attempt recovery? (y/n)")
    if input().upper() != "Y":
        from sys import exit
        logger.fatal("\tRecovery denied! ABORT")
        exit(1)
    logger.error(f"Local file: {temp_archive.name}\n" +
                 f"Remote file: {target_temp_dir}\n" +
                 "Hit [return] when transfer is confirmed.")
    input()
    logger.warn("Transfer recovery complete, continuing.")


def spawn_ssh_process(ssh_host: str) -> Popen:
    from time import sleep
    process = Popen(["ssh", "-tt", ssh_host],
                    stdin=PIPE,
                    stdout=PIPE,
                    bufsize=0,
                    text=True)

    # Hack to wait for user to enter password
    process.stdout.readline()

    return process


def unpack(ssh_process: Popen,
           target_temp_dir: str):
    from multiprocessing import Process

    ssh_process.stdin.write(f"rm -rf {target_temp_dir}_dir \n")
    ssh_process.stdin.write(
        f"7z x -o{target_temp_dir}_dir {target_temp_dir} \n")
    ssh_process.stdin.flush()

    def wait_for_done(ssh_process):
        while True:
            line = ssh_process.stdout.readline()
            if "Everything is Ok" in line:
                return

    p = Process(target=wait_for_done, args=(ssh_process, ))
    p.start()
    p.join(timeout=20)

    if p.is_alive():
        logger.error("Unpack timed out")
        p.kill()
        exit(1)


def move_to_final(ssh_process: Popen,
                  target_dir: str,
                  target_temp_dir: str):
    logger.warn(f"About to delete {target_dir}, ok? (y/n)")
    if input().upper() != "Y":
        logger.fatal("Fail to confirm file deletion, ABORT")
        exit(1)
    ssh_process.stdin.write(f"rm -rf {target_dir} \n")
    ssh_process.stdin.write(f"cp -r {target_temp_dir}_dir {target_dir} \n")
    ssh_process.stdin.write(f"chown -R ubuntu:www {target_dir} \n")
    ssh_process.stdin.write(f"chmod -R 775 {target_dir} \n")
    ssh_process.stdin.write(f"rm -rf {target_temp_dir}" +
                            f" {target_temp_dir}_dir \n")
    ssh_process.stdin.write("bye")
