import logging

from tempfile import TemporaryDirectory, NamedTemporaryFile
from os.path import join, basename, dirname, relpath
from pictures._util import check_valid_archive

logger = logging.getLogger(__name__)


def stage(archive_path: str) -> NamedTemporaryFile:
    if not check_valid_archive(archive_path):
        raise ValueError("Archive not valid. Refusing to deploy.")

    logger.info("Staging started:")

    logger.info("\t- Creating a temporary directory")
    with TemporaryDirectory() as dir:
        unpack(dir, archive_path)
        render(dir, archive_path)
        temp = compress(dir)
    logger.info("Staging complete.\n")
    return temp


def unpack(dir: str, archive_path: str):
    from pictures._util import check_jpg
    from zipfile import ZipFile

    logger.info("\t- Unpacking images")
    with ZipFile(archive_path, 'r') as zip:
        required_files = [f for f in zip.namelist() if check_jpg(f)]
        for file in required_files:
            logger.debug(f"Extracting file: {file}")
            zip.extract(file, path=dir)
    logger.info("\t\tImages successfully unpacked.")


def render(dir: str, archive_path: str):
    from pictures.make_index import make_index
    from static_gen.render import gen_page

    logger.info("\t- Rendering HTML")

    logger.debug("\t\t INDEX GENERATION")
    index = make_index(archive_path, False)

    logger.debug("\t\t RENDERING")
    page = gen_page(index)

    logger.debug("\t\t WRITING OUTPUT")
    with open(join(dir, "index.html"), 'w') as f:
        f.write(page)

    logger.info("\t\t Rendering complete")


def compress(dir: str) -> NamedTemporaryFile:
    from zipfile import ZipFile, ZIP_LZMA
    from glob import glob

    logger.info("\t- Compressing staged directory")

    temp = NamedTemporaryFile(mode='w+b')
    with ZipFile(temp, 'w', ZIP_LZMA) as zip:
        logger.info("\t\t Writing index.html")
        zip.write(join(dir, "index.html"), arcname="index.html")

        logger.info("\t\t Writing image files")
        zip.mkdir("img")
        for image in glob(f"{dir}/**/*.jpg", recursive=True) + \
                glob(f"{dir}/**/*.jpeg", recursive=True):
            corrected_path = join("img",
                                  relpath(image, dir))
            logger.debug(f"\t\tWriting: {corrected_path}")
            zip.write(image, arcname=corrected_path)

    temp.seek(0)  # Reset file pointer
    logger.info("\t\t Compression complete.")

    return temp
